/** PlotLine.java.

	Purpose:
		
	Description:
		
	History:
		6:01:12 PM Jan 13, 2014, Created by RaymondChao

Copyright (C) 2014 Potix Corporation. All Rights Reserved.
 */

package org.zkoss.chart;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.zkoss.chart.OptionDataEvent.EventType;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JSONObject;
import org.zkoss.json.JavaScriptValue;
import org.zkoss.util.CollectionsX;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zk.ui.event.Events;

/**
 * A plot line stretching across the plot area, marking a specific value on one
 * of the axes.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author jumperchen
 * @author RaymondChao
 */
public class PlotLine extends Optionable implements ClientEventHandler {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		color, dashStyle, id, label, value, width, zIndex
	}

	/**
	 * Returns the color of the line.
	 * <p>
	 * Default: null.
	 */
	public Color getColor() {
		return (Color) getAttr(Attrs.color, null).asValue();
	}

	/**
	 * Sets the color of the plot line.
	 */
	public void setColor(Color color) {
		setAttr(Attrs.color, color);
	}

	/**
	 * Sets the color of the plot line.
	 */
	public void setColor(String color) {
		setColor(new Color(color));
	}

	/**
	 * Sets the color of the plot line.
	 */
	public void setColor(LinearGradient color) {
		setColor(new Color(color));
	}

	/**
	 * Sets the color of the plot line.
	 */
	public void setColor(RadialGradient color) {
		setColor(new Color(color));
	}

	/**
	 * Returns the dashing or dot style for the plot line. For possible values
	 * see <a href=
	 * "http://jsfiddle.net/gh/get/jquery/1.7.1/highslide-software/highcharts.com/tree/master/samples/highcharts/plotoptions/series-dashstyle-all/"
	 * > this overview</a>.
	 * <p>
	 * Default: "Solid".
	 */
	public String getDashStyle() {
		return getAttr(Attrs.dashStyle, "Solid").asString();
	}

	/**
	 * Sets the dashing or dot style for the plot line. For possible values see
	 * <a href=
	 * "http://jsfiddle.net/gh/get/jquery/1.7.1/highslide-software/highcharts.com/tree/master/samples/highcharts/plotoptions/series-dashstyle-all/"
	 * > this overview</a>.
	 */
	public void setDashStyle(String dashStyle) {
		if (!("Solid".equalsIgnoreCase(dashStyle) || "ShortDash".equalsIgnoreCase(dashStyle)
				|| "ShortDot".equalsIgnoreCase(dashStyle)
				|| "ShortDashDot".equalsIgnoreCase(dashStyle)
				|| "ShortDashDotDot".equalsIgnoreCase(dashStyle)
				|| "Dot".equalsIgnoreCase(dashStyle) || "Dash".equalsIgnoreCase(dashStyle)
				|| "LongDash".equalsIgnoreCase(dashStyle) || "DashDot".equalsIgnoreCase(dashStyle)
				|| "LongDashDot".equalsIgnoreCase(dashStyle) || "LongDashDotDot"
					.equalsIgnoreCase(dashStyle)))
			throw new IllegalArgumentException("Unsupported style: ["
					+ dashStyle + "]");
		setAttr(Attrs.dashStyle, dashStyle, "Solid");
	}

	/**
	 * Returns an id used for identifying the plot line in
	 * {@link Axis#removePlotLine(String)}
	 * <p>
	 * Default: null.
	 */
	public String getId() {
		return getAttr(Attrs.id, null).asString();
	}

	/**
	 * Sets an id used for identifying the plot line in
	 * {@link Axis#removePlotLine(String)}
	 * <p>
	 * Default: null.
	 */
	public void setId(String id) {
		setAttr(Attrs.id, id);
	}

	/**
	 * Returns text labels for the plot line
	 * <p>
	 * Default: an instance of PlotLabel.
	 */
	public PlotLabel getLabel() {
		PlotLabel label = (PlotLabel) this.getAttr(Attrs.label);

		if (label == null) {
			label = new PlotLabel();
			setLabel(label);
		}
		return label;
	}

	/**
	 * Sets text labels for the plot line
	 */
	public void setLabel(PlotLabel label) {
		setAttr(Attrs.label, label);
	}

	/**
	 * Returns the position of the line in axis units.
	 * <p>
	 * Default: null.
	 */
	public Number getValue() {
		return getAttr(Attrs.value, null).asNumber();
	}

	/**
	 * Sets the position of the line in axis units.
	 */
	public void setValue(Number value) {
		setAttr(Attrs.value, value);
	}

	/**
	 * Returns the width or thickness of the plot line.
	 * <p>
	 * Default: null.
	 */
	public Number getWidth() {
		return getAttr(Attrs.width, null).asNumber();
	}

	/**
	 * Sets the width or thickness of the plot line.
	 */
	public void setWidth(Number width) {
		setAttr(Attrs.width, width);
	}

	/**
	 * Returns the z index of the plot line within the chart.
	 * <p>
	 * Default: null.
	 */
	public Number getZIndex() {
		return getAttr(Attrs.zIndex, null).asNumber();
	}

	/**
	 * Sets the z index of the plot line within the chart.
	 */
	public void setZIndex(Number zIndex) {
		setAttr(Attrs.zIndex, zIndex);
	}
	
	private Map<String, List<EventListener<? extends Event>>> _events;

	/**
	 * Adds an event listener to this options.
	 * @param evtnm what event to listen (never null), i.e. onClick, onMouseOver, and so on.
	 * @since 1.2.0
	 */
	public void addEventListener(String evtnm,
			EventListener<? extends Event> listener) {
		if (!Events.isValid(evtnm))
			throw new IllegalArgumentException("Invalid event name: "+evtnm);
		if (_events == null)
			_events = new HashMap<String, List<EventListener<? extends Event>>>(
					3);
		
		List<EventListener<? extends Event>> list = _events.get(evtnm);
		if (list == null) {
			list = new LinkedList<EventListener<? extends Event>>();
			list.add(listener);
			_events.put(evtnm, list);
		} else {
			list.add(listener);
		}
		fireEvent(EventType.REGISTER, evtnm, getEventListeners(evtnm), null);
	}
	
	/**
	 * Removes an event listener.
	 * @return whether the listener is removed; false if it was never added.
	 * @since 1.2.0
	 */
	public boolean removeEventListener(String evtnm, EventListener<? extends Event> listener) {
		if (evtnm == null || listener == null)
			throw new IllegalArgumentException("null");
		List<EventListener<? extends Event>> list = _events == null ? null : _events.get(evtnm);
		if (list != null) {
			for (Iterator<EventListener<? extends Event>> it = list.iterator(); it.hasNext();) {
				EventListener<? extends Event> next = it.next();
				if (next.equals(listener))
					it.remove();
				if (list.isEmpty()) {
					_events.remove(evtnm);
					fireEvent(EventType.UNREGISTERED, evtnm, null, null);
				} else {
					// update list
					fireEvent(EventType.REGISTER, evtnm, getEventListeners(evtnm), null);
				}

				return true;
			}
		}
		return false;
	}
	/**
	 * Returns all event names, if any.
	 * @since 1.2.0
	 */
	@SuppressWarnings("unchecked")
	public Set<String> getEventNames() {
		if (_events == null)
			return (Set<String>) Collections.EMPTY_SET;
		return _events.keySet();
	}
	/** Returns an iterative collection of the event listeners for the given event.
	 * <p>Note: it is OK to invoke {@link #addEventListener} or {@link #removeEventListener}
	 * when iterating through the event listeners with the returned collection.
	 * <p>To remove an event listener from the returned iterative collection,
	 * you could invoke {@link Iterable#iterator}'s {@link Iterator#remove}.
	 * @since 1.2.0
	 */
	public Iterable<EventListener<? extends Event>> getEventListeners(String evtnm) {
		final List<EventListener<? extends Event>> lis = _events == null ? null :_events.get(evtnm);
		if (lis != null)
			return new Iterable<EventListener<? extends Event>>() {
				public Iterator<EventListener<? extends Event>> iterator() {
					return CollectionsX.comodifiableIterator(lis);
				}
			};
		return CollectionsX.emptyIterable();
	}
	
	public String toJSONString() {
		if (_events == null || _events.isEmpty()) {
			return JSONObject.toJSONString(options);
		} else {
			HashMap<String, Object> clone = new HashMap<String, Object>(options);
			clone.put("uuid", hashCode());
			JSONObject events = new JSONObject();
			clone.put("events", events);
			for (String nm : getEventNames()) {
				nm = nm.substring(2).toLowerCase();
				events.put(
						nm,
						new JavaScriptValue(
								"function (e) {var p = this.axis.chart.pointer; if (!p || !p.hasDragged) zk.Widget.$(e.target).sendDomEvent(e, this);}"));
			}
			return JSONObject.toJSONString(clone);
		}
	}
}